;;; bep.el --- Bilingual extension to emacspeak
;;; $Id: bep.el,v 1.6 2001/10/08 07:52:16 inoue Exp $
;;; $Author: inoue $
;;; Description: Contains functions that handle Japanese characters
;;; Keywords: Emacspeak, Japanese, multilingualization
;;;
;;; This file is not part of Emacspeak

;;;{{{  Copyright:
;;;Copyright (C) 2000, Takayuki Watanabe (takayuki@la.shonan-it.ac.jp)
;;; All Rights Reserved.
;;;
;;; This file is not part of GNU Emacs, but the same permissions apply.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;;}}}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;{{{  Required modules
;; phonetic table for Japanese characters
(require 'auditory-disp-table-jp)
(defvar bep t "Bilingual extension to emacspeak")

;;}}}
;;{{{  Introduction:

;;; Commentary:

;;; BEP, Bilingual Emacspeak Project, extends Emacspeak to bilingual, 
;;; Japanese and English.
;;; Japanese characters shoud be emcoded with Shift JIS.
;;; ShiftJIS is double-byte character.

;;}}}
;;{{{ bep functions

;(defun bep-get-phonetic-string-E (char-string)
;  "Return the phonetic string for this CHAR or its upper case equivalent."
;(interactive "sChar:\n") 
;(or (cdr
;     (assoc char-string voice-char-to-phonetic-table-E ))
;    " "))

;(defun bep-get-phonetic-string (char)
;  "Return the phonetic string for this CHAR or its upper case equivalent."
;(interactive "sChar:\n") 
;(or (cdr
;     (assoc char bep-char-to-phonetic-table-J ))
;    " "))

(defvar emacspeak-jp-type-prefix-table '(
(jp-hiragana . "$B%R%i%,%J(B")
(jp-katakana . "$B%+%?%+%J(B")
(jp-alpha . "$B%<%s%+%/%m!<%^%8(B")
(katakana . "$B%O%s%+%/(B")
(jp-other . "$B%<%s%+%/(B")
))

(defvar emacspeak-jp-subtype-prefix-table '(
(upcase . "$B%*%*%b%8(B")
(small . "$B%A%$%5%$(B")
))

(defvar emacspeak-jp-small-char-list
'(?$B$!(B ?$B$#(B ?$B$%(B ?$B$'(B ?$B$)(B ?$B$C(B ?$B$c(B ?$B$e(B ?$B$g(B ?$B%!(B ?$B%#(B ?$B%%(B ?$B%'(B ?$B%)(B ?$B%C(B ?$B%c(B ?$B%e(B ?$B%g(B
      ?(I1(B ?(I2(B ?(I3(B ?(I4(B ?(I5(B ?(IT(B ?(IU(B ?(IV(B))

(defun emacspeak-jp-convert-string-to-phonetic (str)
  (let ((cur 0)
	(cidx 0)
	(siz (length str))
	(ret ""))
    (while (< cur siz)
      (let* ((c (aref str cur))
	     (ct (emacspeak-jp-get-char-type c))
	ptr)
	;;; get the end of region with current char type
	(setq cidx (1+ cur))
	(while (and (< cidx siz)
		    (eq ct (emacspeak-jp-get-char-type
			    (aref str cidx))))
	  (setq cidx (1+ cidx)))
	;;; make phonetic string
	(setq ret (concat ret
			  (or (cdr
			       (assoc ct emacspeak-jp-type-prefix-table))
			       "")
			   ))
	(setq ptr cur)
	(while (< ptr cidx)
	  (let* ((cptr (aref str ptr))
		 (cstr (char-to-string cptr))
		 (phon (if (memq ct '(jp-hiragana jp-katakana katakana jp-alpha alpha))
			   (bep-get-cursor-string cstr)
			   (bep-get-explanatory-string cstr)))
		 (subtype (emacspeak-jp-get-char-subtype cptr))
		 (pref (or (cdr (assoc subtype emacspeak-jp-subtype-prefix-table)) ""))
		 )
	    (setq ret (concat ret pref phon)))
	  (if (and (< ptr (1- siz)) (eq ct 'jp-kanji))
	      (setq ret (concat ret " ")))
	  (setq ptr (1+ ptr))
	  )
	(setq cur cidx)
	)
      )
    ret
    )
)

(defun emacspeak-jp-convert-char-to-phonetic (cstr)
  "convert first character of string to explanatory string with prefix indicating character subtype."
  (let* ((cptr (aref cstr 0))
	 (phon (bep-get-explanatory-string cstr))
	 (ct (emacspeak-jp-get-char-type cptr))
	 (subtype (emacspeak-jp-get-char-subtype cptr))
	 (pref 
	  (or (cdr
	       (assoc ct emacspeak-jp-type-prefix-table))
	      "")
	  )
	 (spref (or (cdr (assoc subtype emacspeak-jp-subtype-prefix-table)) ""))
	 ret)
    (setq ret (concat ret pref spref phon))
    ret)
)

(defun emacspeak-jp-convert-string-to-cursor (str)
  "convert string to concatenated string of cursor-string"
  (let ((siz (length str))
	     (output "")
	     (ptr 0))
    (while (< ptr siz)
      (setq cstr (substring str ptr (1+ ptr)))
      (setq output (concat output (bep-get-cursor-string cstr)))
      (setq ptr (1+ ptr))
      )
  output
))

(defun emacspeak-jp-get-char-type (c)
  "returns Japanese character type"
  (let ((ct (bep-get-char-type c))
	(cstr (char-to-string c)))
  (cond
   (ct ct) ; if char-type is defined in auditory-disp-table-jp
   ((string-match "[0-9]" cstr) 'number)
   ((string-match "[A-Za-z]" cstr) 'alpha)
   ((string-match "[(I&(B-(I_(B]" cstr) 'katakana)
   (t nil))
))

(defun emacspeak-jp-get-char-subtype (c)
  "returns subtype of c"
  (cond
   ((or
     (and (>= c ?A) (<= c ?Z))
     (and (>= c ?$B#A(B) (<= c ?$B#Z(B))
     (and (>= c ?$B&!(B) (<= c ?$B&8(B)))
    'upcase)
   ((memq c emacspeak-jp-small-char-list)
    'small)
   (t nil)
   )
)

(defun bep-is-Japanese-p (char-string)
  "return True if Japanese, i.e. double-byte character"
(interactive)
   (equal 0 (string-match "\\cj" char-string)) ; if Japanese characters
)

;;}}}
(provide 'bep)
;;{{{ end of file

;;; local variables:
;;; folded-file: t
;;; byte-compile-dynamic: t
;;; end:

;;}}}



