;;; bep-util.el --- Utility functions for BEP

;; Copyright (C) 2001  Free Software Foundation, Inc.

;; Author: Koichi INOUE <inoue@argv.org>
;; Keywords: data, languages

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Utility functions for BEP.
;; bep-dict-* is for conversion of english-katakana dictionary from and to
;; the original Grassroots format. It is simply zenkaku-hankaku transfer.

;;; Code:

(defun bep-convert-kana-han2zen-region (beg end)
  "Convert katakana-jisx0201 in region to jisx0208."
  (interactive "r")
  (goto-char beg)
  (let (mbeg mend
	     (e20-p (eq emacs-major-version 20))
	     (cnt 0)
	     (hankaku-long-tone
	      (char-to-string (make-char 'katakana-jisx0201 176)))
	     (zenkaku-long-tone
	      (char-to-string (make-char 'japanese-jisx0208 33 60)))
	     )
    (while (re-search-forward "\\ck+" end t) ; hankaku kana
      (setq cnt (1+ cnt))
      (setq mbeg (match-beginning 0)
	    mend (match-end 0))
      (japanese-zenkaku-region mbeg mend)
      ; There is a bug in Emacs-20's conversion table.
      (and e20-p
	   (progn
	     (goto-char mbeg)
	     (while (re-search-forward hankaku-long-tone mend t)
	       (replace-match zenkaku-long-tone))
	     (goto-char mend)))
    )
    (message (format "replaced %s parts" cnt))
cnt))

(defun bep-convert-kana-zen2han-region (beg end)
  "Convert katakana-jisx0201 in region to jisx0208."
  (interactive "r")
  (goto-char beg)
  (let (mbeg mend
	     (mkend (copy-marker end))
	     (e20-p (eq emacs-major-version 20))
	     (cnt 0)
	     (hankaku-long-tone
	      (char-to-string (make-char 'katakana-jisx0201 176)))
	     )
    (while (re-search-forward "\\cK+" (marker-position mkend) t) ; zenkaku kana
      (setq cnt (1+ cnt))
      (setq mbeg (match-beginning 0)
	    mend (match-end 0))
      (japanese-hankaku-region mbeg mend)
      ; There is a bug in Emacs-20's conversion table.
      (and e20-p
	   (progn
	     (goto-char mbeg)
	     (while (re-search-forward "-" mend t)
	       (replace-match hankaku-long-tone))
	     (goto-char mend)))
    )
    (message (format "replaced %s parts" cnt))
cnt))

(defun bep-dict-convert-org-to-bep (filename)
  "Read English to kana dictionary and convert hankaku kana to zenkaku"
  (interactive "fFile to convert: ")
  (let ((buf (get-buffer-create "*BEP Conversion*")))
    (set-buffer buf)
    (buffer-disable-undo)
    (erase-buffer)
    (let ((coding-system-for-read 'japanese-shift-jis))
      (insert-file-contents filename))
    (goto-char (point-min))
    (switch-to-buffer buf)
    (message "Converting...")
    (bep-convert-kana-han2zen-region (point-min) (point-max))
    (message "Converting...done")
    )
  (goto-char (point-min))
  (message "Now save this buffer with C-x C-w")
)

(defun bep-dict-convert-bep-to-org (filename)
  "Read English to kana dictionary and convert zenkaku katakana to hankaku"
  (interactive "fFile to convert: ")
  (let ((buf (get-buffer-create "*BEP Conversion*")))
    (set-buffer buf)
    (buffer-disable-undo)
    (erase-buffer)
    (let ((coding-system-for-read 'japanese-shift-jis))
      (insert-file-contents filename))
    (goto-char (point-min))
    (switch-to-buffer buf)
    (message "Converting...")
    (bep-convert-kana-zen2han-region (point-min) (point-max))
    (message "Converting...done")
    )
  (goto-char (point-min))
  (message "Now save this buffer with C-x C-w")
)

(provide 'bep-util)
;;; bep-util.el ends here
